#!/usr/bin/python
# -*- coding: utf-8 -*-   
#
#  test_simple.py
#  
#
#  Created by TVA on 1/28/18.
#  Copyright (c) 2018 parseyml. All rights reserved.
#
from __future__ import unicode_literals
from cStringIO import StringIO
import sys
import re
from parseyml import main
import os

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


def test_case1():
    sys.stdout = mystdout = StringIO()

    main(['parseyml', os.path.join(BASE_DIR, 'test/test_input/docker-compose.yml'), 'COMPOSE'])

    result_stdout = mystdout.getvalue()
    assert len(result_stdout) > 0
    assert "export COMPOSE__SERVICES__WEORDER__IMAGE='registry.gitlab.com/tranvietanh1991/weorder:weoder-restapi-v1.2'" in result_stdout
    assert "export COMPOSE__SERVICES__MYSQLDB__ENVIRONMENT__MYSQL_ROOT_PASSWORD='dummy_password'" in result_stdout
    mystdout.close()


def test_case2():
    with open(os.path.join(BASE_DIR, 'test/test_input/docker-compose.yml')) as f:
        sys.stdout = mystdout = StringIO()
        sys.stdin = StringIO(f.read())
        main(['parseyml', 'COMPOSE'])

        result_stdout = mystdout.getvalue()
        assert len(result_stdout) > 0
        assert "export COMPOSE__SERVICES__WEORDER__IMAGE='registry.gitlab.com/tranvietanh1991/weorder:weoder-restapi-v1.2'" in result_stdout
        assert "export COMPOSE__SERVICES__MYSQLDB__ENVIRONMENT__MYSQL_ROOT_PASSWORD='dummy_password'" in result_stdout
        mystdout.close()


def test_case3():
    sys.stdout = mystdout = StringIO()
    os.environ.setdefault('BUILD_VERSION', '1.2.3')
    main(['parseyml', os.path.join(BASE_DIR, 'test/test_input/docker-compose.build.yml'), 'COMPOSE'])

    result_stdout = mystdout.getvalue()
    assert len(result_stdout) > 0
    assert "export COMPOSE__SERVICES__DEVELOP__IMAGE='posttrade_api:dev_1.2.3'" in result_stdout
    assert "export COMPOSE__SERVICES__PRODUCTION__IMAGE='posttrade_api:1.2.3'" in result_stdout
    mystdout.close()



#!/usr/bin/python
# -*- coding: utf-8 -*-   
#
#  __init__.py
#  
#
#  Created by TVA on 1/25/18.
#  Copyright (c) 2018 parseyml. All rights reserved.
#
from parse_yml import main

__all__ = ['main', 'travel_and_print_env']


# !/usr/bin/python
# -*- coding: utf-8 -*-
#
#  parse_yml.py
#
#
#  Created by TVA on 6/17/16.
#  Copyright (c) 2016 parseyml. All rights reserved.
#
from __future__ import unicode_literals
import os, sys, yaml, re

# Setup YAML parser to check for shell enviroment substituting

ENV_VAR_MATCHER = re.compile(
    r"""
        \$\{       # match characters `${` literally
        ([^}:\s]+) # 1st group: matches any character except `}` or `:`
        :?         # matches the literal `:` character zero or one times
        ([^}]+)?   # 2nd group: matches any character except `}`
        \}         # match character `}` literally
    """, re.VERBOSE
)
IMPLICIT_ENV_VAR_MATCHER = re.compile(
    r"""
        .*          # matches any number of any characters
        \$\{.*\}    # matches any number of any characters
                    # between `${` and `}` literally
        .*          # matches any number of any characters
    """, re.VERBOSE
)


def _replace_env_var(match):
    env_var, default = match.groups()
    return os.environ.get(env_var, default)


def env_var_constructor(loader, node):
    raw_value = loader.construct_scalar(node)
    value = ENV_VAR_MATCHER.sub(_replace_env_var, raw_value)
    return yaml.safe_load(value)


def setup_yaml_parser():
    yaml.add_constructor('!env_var', env_var_constructor)
    yaml.add_implicit_resolver('!env_var', IMPLICIT_ENV_VAR_MATCHER)


# Main logic

def travel_and_print_env(data, root_key=''):
    if isinstance(data, dict):
        for key in data:
            if root_key:
                cur_key = root_key + '__' + key
            item = data[key];
            if isinstance(item, dict) or isinstance(item, list):
                travel_and_print_env(item, root_key=cur_key);
            else:
                sys.stdout.write("export %s='%s'\n" % (cur_key.upper(), item))
    else:
        for i in range(len(data)):
            cur_key = root_key + '_%s_' % i
            item = data[i]
            if isinstance(item, dict) or isinstance(item, list):
                travel_and_print_env(item, root_key=cur_key);
            else:
                sys.stdout.write("export %s='%s'\n" % (cur_key.upper(), item))


def validate_root_key(root_key):
    m = re.match(r'^[a-zA-Z]\w*$', root_key)
    return m is not None


def main(argv):
    setup_yaml_parser()
    args = argv[1:]
    if len(args) == 2:  # read yml from file
        filePath, root_key = args
        with open(filePath) as f:
            data = yaml.load(f);
            if not validate_root_key(root_key):
                sys.stderr.write("invalid root_key example: 'ROOT_KEY'")
                return 1
            travel_and_print_env(data, root_key=root_key.strip().upper());
    elif len(args) == 1:  # read yml from pipe line
        root_key = args[0]
        if not validate_root_key(root_key):
            sys.stderr.write("invalid root_key example: 'ROOT_KEY'")
            return 1
        data = yaml.load(sys.stdin.read())
        travel_and_print_env(data, root_key=root_key.strip().upper());

    return 0


if __name__ == '__main__':
    result = main(sys.argv)
    if result == 0:
        sys.stdout.write("# Run this command to configure your shell:\n")
        sys.stdout.write("# eval $(python %s)\n" % ' '.join(sys.argv))
    sys.exit(result)



#!/usr/bin/python
# -*- coding: utf-8 -*-   
#
#  setup.py
#  
#
#  Created by TVA on 1/25/18.
#  Copyright (c) 2018 parseyml. All rights reserved.
#
from __future__ import unicode_literals
from setuptools import setup, find_packages
import os

with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'parseyml/VERSION')) as f:
    __version__ = f.read()

setup(
    name='parseyml',
    version=__version__,
    description='A python executable script to parse yml file and export to shell environment',
    author='V.Anh Tran',
    author_email='tranvietanh1991@gmail.com',
    license="MIT",
    url='https://github.com/tranvietanh1991/parseyml',  # use the URL to the github repo
    download_url='https://github.com/tranvietanh1991/parseyml/archive/master.zip',  # source code download
    packages=find_packages(exclude=['*.tests', '*.tests.*']),
    scripts=['bin/parseyml'],
    include_package_data=True,
    install_requires=[
        "PyYAML>=3.11",
    ],
    python_requires='>=2.6, <3',
    keywords=['yaml', 'parse', 'parseyml', 'parse-yml', 'yml', 'export', 'shell'],  # arbitrary keywords
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        # "Framework :: Django",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        # "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
